#
# SSA: 1D Singular Spectrum Analysis for snr enhancement.
#  
# (dp, sing) = ssa(d, nw, p);
#
# Inputs:    d         1D time series (column).
#            nw        view used to make the Hankel matrix.
#            p         number of singular values used to reconstruct the data.
#
# Outputs:   dp        predicted (clean) data.
#            sing      singular values of the Hankel matrix.
#
# Example:
# 
#  d = cos(2.0*pi*0.01*[1: 1: 200]) + 0.5 * randn(1, 200);
#  d = d[:];
#  (dp, sing) = ssa(d, 100, 2, 1);
#  using Winston;
#  figure();
#  plot(d, "k");
#  oplot(dp+3, "b");
#
include("average_anti_diag.jl");

function ssa(d, nw, p)
 
 d  = d[:];
 nt = length(d);
 N  = nt - nw + 1;
 l  = (1: 1: nw);
 M  = zeros(length(l), N) + im * zeros(length(l), N);

 # Make Hankel matrix:
 for k = 1: N
     M[:, k] = d[k+l-1, 1];
 end

 # Eigenimage decomosition:
 (U, S, V) = svd(M, thin=false);

 # Reconstruct with one oscullatory component at the time:
 Mp = zeros(size(M));
 for k = 1: p
     u  = U[:, k];
     Mp = Mp + (u*u') * M;
 end;
 dp   = average_anti_diag(Mp);
 sing = S;

 # Outputs: 
 return dp, sing;
end 
